// This is a scripted coppercube action. It starts/stops dragging a 3d object with the mouse. 
// Add this action into the "when clicked onto this do something" behavior to make it dragable.
// It is possible to constrain the drag movement to an axis by deactivating the 'x', 'y' and 'z' flags.
//
// The following embedded xml is for the editor and describes how the behavior can be edited:
// Supported types are: int, float, string, bool, color, vect3d, scenenode, texture, action
/*
	<action jsname="action_Drag3DObject_axis" description="Drag 3D Object with axis constraints">
		<property name="x" type="bool" default="true" />
		<property name="y" type="bool" default="true" />
		<property name="z" type="bool" default="true"/>
	</action>
*/

action_Drag3DObject_axis = function()
{
};

currentlyDraggingAction = null;
dontRestartDraggingUntil = 0;

// called when the action is executed 
action_Drag3DObject_axis.prototype.execute = function(currentNode)
{
	if (dontRestartDraggingUntil != 0)
	{
		var now = (new Date()).getTime();
		if (dontRestartDraggingUntil > now)
			return;
	}
	
	if (currentlyDraggingAction)
	{
		currentlyDraggingAction.stopDragging();
		return;
	}
			
	currentlyDraggingAction = this;
	
	var me = this; 
	this.registeredFunction = function() { me.draggingFunc(); }; 
	
	ccbRegisterOnFrameEvent(this.registeredFunction);	
	
	this.currentNode = currentNode;
	
	var positionNode = ccbGetSceneNodeProperty(currentNode, "Position");
	var positionCam =  ccbGetSceneNodeProperty(ccbGetActiveCamera(), "Position");	
	
	this.distance = positionNode.substract(positionCam).getLength();
}


action_Drag3DObject_axis.prototype.stopDragging = function()
{
	this.currentNode = null;
	ccbUnregisterOnFrameEvent(this.registeredFunction);
	currentlyDraggingAction = null;
	
	dontRestartDraggingUntil = (new Date()).getTime() + 300;
}


action_Drag3DObject_axis.prototype.draggingFunc = function()
{
	// update position
	
	var mouseX = ccbGetMousePosX();
	var mouseY = ccbGetMousePosY();

	var pos3d = ccbGet3DPosFrom2DPos(mouseX, mouseY);
	var positionCam =  ccbGetSceneNodeProperty(ccbGetActiveCamera(), "Position");	
	var positionObj =  ccbGetSceneNodeProperty(this.currentNode, "Position");	
	
	var ray = pos3d.substract(positionCam);
	ray.normalize();
	ray.x *= this.distance;
	ray.y *= this.distance;
	ray.z *= this.distance;
	
	var newposx = ray.x + positionCam.x;
	var newposy = ray.y + positionCam.y;
	var newposz = ray.z + positionCam.z;
	
	if (!this.x) newposx = positionObj.x;
	if (!this.y) newposy = positionObj.y;
	if (!this.z) newposz = positionObj.z;	
	
	ccbSetSceneNodeProperty(this.currentNode, "Position", newposx, newposy, newposz);
}
